""" Binance Spot Broker """

from pricehub.brokers.broker_binance import BrokerBinanceABC


class BrokerBinanceSpot(BrokerBinanceABC):
    """Binance Spot Broker"""

    base_url = "https://api.binance.com/api/v3/klines"



""" Binance Futures Broker Class """

from pricehub.brokers.broker_binance import BrokerBinanceABC


class BrokerBinanceFutures(BrokerBinanceABC):
    """
    Binance Futures Broker Class
    """

    base_url = "https://fapi.binance.com/fapi/v1/klines"



""" Abstract base class for brokers """

from abc import ABC
import pandas as pd


class BrokerABC(ABC):
    """
    Abstract base class for brokers
    """

    def get_ohlc(self, get_ohlc_params: "GetOhlcParams") -> pd.DataFrame:  # type: ignore[name-defined]
        """
        Get OHLC data from the broker.
        :param get_ohlc_params:
        :return:
        """
        raise NotImplementedError



""" Broker Enum class """

from enum import Enum

from pricehub.brokers.broker_binance_futures import BrokerBinanceFutures
from pricehub.brokers.broker_binance_spot import BrokerBinanceSpot
from pricehub.brokers.broker_bybit_spot import BrokerBybitSpot


class Broker(Enum):
    """
    Broker Enum class
    """

    BINANCE_SPOT = "binance_spot"
    BINANCE_FUTURES = "binance_futures"
    BYBIT_SPOT = "bybit_spot"

    def get_broker_class(self) -> "BrokerABC":  # type: ignore[name-defined]
        """
        Get the broker class for the broker.
        :return:
        """
        broker_classes = {
            Broker.BINANCE_SPOT: BrokerBinanceSpot,
            Broker.BINANCE_FUTURES: BrokerBinanceFutures,
            Broker.BYBIT_SPOT: BrokerBybitSpot,
        }
        return broker_classes[self]



from setuptools import setup, find_packages


package_name = "pricehub"


setup(
    name=package_name,
    version="0.0.1",
    packages=find_packages(),
    author="Evgenii Lazarev",
    author_email="elazarev@gmail.com",
    project_urls={
        "LinkedIn": "https://www.linkedin.com/in/elazarev",
    },
    description="Open-High-Low-Close (OHLC) prices data from different brokers",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "pandas",
        "pydantic>=2.0.1",
        "arrow>=1.0.0",
        "requests",
    ],
)



""" Binance Broker ABC Class """

import requests
import pandas as pd

from pricehub.brokers.broker_abc import BrokerABC
from pricehub.config import TIMEOUT_SEC


class BrokerBinanceABC(BrokerABC):
    """
    Binance Broker  ABC Class
    """

    base_url = ""
    columns = [
        "Open time",
        "Open",
        "High",
        "Low",
        "Close",
        "Volume",
        "Close time",
        "Quote asset volume",
        "Number of trades",
        "Taker buy base asset volume",
        "Taker buy quote asset volume",
        "Ignore",
    ]

    def get_ohlc(self, get_ohlc_params: "GetOhlcParams") -> pd.DataFrame:  # type: ignore[name-defined]
        start_time = int(get_ohlc_params.start.timestamp() * 1000)
        end_time = int(get_ohlc_params.end.timestamp() * 1000)

        all_data = []

        while start_time < end_time:
            url = (
                f"{self.base_url}?symbol={get_ohlc_params.symbol}&interval={get_ohlc_params.interval}"
                f"&startTime={start_time}&endTime={end_time}"
            )
            data = requests.get(url, timeout=TIMEOUT_SEC).json()
            if not data:
                break
            all_data.extend(data)
            start_time = data[-1][0] + 1

        df = pd.DataFrame(
            all_data,
            columns=self.columns,
        )
        df = df.astype(float)
        df["Open time"] = pd.to_datetime(df["Open time"], unit="ms")
        df["Close time"] = pd.to_datetime(df["Close time"], unit="ms")
        df.set_index("Open time", inplace=True)

        return df
